from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.realm_repository_abc import RealmRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.realm import Realm
from bot_data.model.realm_moderator import RealmModerator


class RealmRepositoryService(RealmRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context
        self._servers = servers
        self._users = users

        RealmRepositoryABC.__init__(self)

    def _from_result(self, result: tuple) -> Realm:
        return Realm(
            result[1],
            result[2],
            self._servers.get_server_by_id(result[3]),
            result[4],
            result[5],
            id=result[0],
        )

    def get_all(self) -> List[Realm]:
        self._logger.trace(__name__, f"Send SQL command: {Realm.get_select_all_string()}")
        return List(
            Realm, [self._from_result(result) for result in self._context.select(Realm.get_select_all_string())]
        )

    def get_realm_by_id(self, id: int) -> Realm:
        self._logger.trace(__name__, f"Send SQL command: {Realm.get_select_by_id_string(id)}")
        return self._from_result(self._context.select(Realm.get_select_by_id_string(id))[0])

    def find_realm_by_id(self, id: int) -> Optional[Realm]:
        self._logger.trace(__name__, f"Send SQL command: {Realm.get_select_by_id_string(id)}")
        result = self._context.select(Realm.get_select_by_id_string(id))
        if result is None or len(result) == 0:
            return None

        return self._from_result(result[0])

    def find_realm_by_name(self, name: str) -> Optional[Realm]:
        self._logger.trace(__name__, f"Send SQL command: {Realm.get_select_by_name_string(name)}")
        result = self._context.select(Realm.get_select_by_name_string(name))
        if result is None or len(result) == 0:
            return None

        return self._from_result(result[0])

    def get_realms_by_server_id(self, id: int) -> List[Realm]:
        self._logger.trace(__name__, f"Send SQL command: {Realm.get_select_by_server_id_string(id)}")
        return List(
            Realm,
            [self._from_result(result) for result in self._context.select(Realm.get_select_by_server_id_string(id))],
        )

    def add_realm(self, realm: Realm):
        self._logger.trace(__name__, f"Send SQL command: {realm.insert_string}")
        self._context.cursor.execute(realm.insert_string)

    def update_realm(self, realm: Realm):
        self._logger.trace(__name__, f"Send SQL command: {realm.udpate_string}")
        self._context.cursor.execute(realm.udpate_string)

    def delete_realm(self, realm: Realm):
        self._logger.trace(__name__, f"Send SQL command: {realm.delete_string}")
        self._context.cursor.execute(realm.delete_string)

    def _rule_from_result(self, result: tuple) -> RealmModerator:
        return RealmModerator(
            self.get_realm_by_id(result[1]),
            self._users.get_user_by_id(result[2]),
            result[4],
            result[5],
            id=result[0],
        )

    def get_realm_moderators(self) -> List[RealmModerator]:
        self._logger.trace(__name__, f"Send SQL command: {RealmModerator.get_select_all_string()}")
        return List(
            RealmModerator,
            [self._rule_from_result(result) for result in self._context.select(RealmModerator.get_select_all_string())],
        )

    def get_realm_moderator_by_id(self, id: int) -> RealmModerator:
        self._logger.trace(__name__, f"Send SQL command: {RealmModerator.get_select_by_id_string(id)}")
        return self._rule_from_result(self._context.select(RealmModerator.get_select_by_id_string(id))[0])

    def get_realm_moderators_by_realm_id(self, id: int) -> List[RealmModerator]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {RealmModerator.get_select_by_realm_id_string(id)}",
        )
        return List(
            RealmModerator,
            [
                self._rule_from_result(result)
                for result in self._context.select(RealmModerator.get_select_by_realm_id_string(id))
            ],
        )

    def add_realm_moderator(self, realm_moderator: RealmModerator):
        self._logger.trace(__name__, f"Send SQL command: {realm_moderator.insert_string}")
        self._context.cursor.execute(realm_moderator.insert_string)

    def update_realm_moderator(self, realm_moderator: RealmModerator):
        self._logger.trace(__name__, f"Send SQL command: {realm_moderator.udpate_string}")
        self._context.cursor.execute(realm_moderator.udpate_string)

    def delete_realm_moderator(self, realm_moderator: RealmModerator):
        self._logger.trace(__name__, f"Send SQL command: {realm_moderator.delete_string}")
        self._context.cursor.execute(realm_moderator.delete_string)
