from typing import List as TList
from typing import Optional

import discord
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord import app_commands, Member
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.abc.permission_service_abc import PermissionServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.realm_repository_abc import RealmRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.realm import Realm
from bot_data.model.realm_moderator import RealmModerator
from bot_data.model.technician_config import TechnicianConfig
from modules.realms.realm_utils import RealmUtils


class RealmGroup(DiscordCommandABC):
    def __init__(
        self,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        bot: DiscordBotServiceABC,
        client_utils: ClientUtilsABC,
        translate: TranslatePipe,
        permissions: PermissionServiceABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        realms: RealmRepositoryABC,
        settings: TechnicianConfig,
        realm_utils: RealmUtils,
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._t = translate
        self._permissions = permissions
        self._servers = servers
        self._users = users
        self._realms = realms
        self._settings = settings
        self._realm_utils = realm_utils

        self._logger.trace(__name__, f"Loaded command service: {type(self).__name__}")

    async def _auto_realm_complete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        server = self._servers.get_server_by_discord_id(interaction.guild.id)
        realms = self._realms.get_realms_by_server_id(server.id).select(lambda x: x.name)
        return [
            app_commands.Choice(name=realm, value=realm)
            for realm in self._client_utils.get_auto_complete_list(realms, current, lambda x: x.name)
        ]

    @commands.hybrid_group(name="realm")
    @commands.guild_only()
    async def realm(self, ctx: Context):
        pass

    @realm.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def list(self, ctx: Context):
        self._logger.debug(__name__, f"Received command realm list {ctx}")
        realms = self._realms.get_all()

        if len(realms) == 0:
            await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.realm.list.error.nothing_found"))
            self._logger.trace(__name__, f"Finished command realm list")
            return

        embed = discord.Embed(
            title=self._t.transform("modules.realm.list.title"),
            description=self._t.transform("modules.realm.list.description"),
            color=int("ef9d0d", 16),
        )

        realm_names = ""
        for realm in realms:
            realm_names += f"\n{realm.name}"

        embed.add_field(
            name=self._t.transform("modules.realm.list.realm_names"),
            value=realm_names,
            inline=True,
        )

        await self._message_service.send_ctx_msg(ctx, embed)
        self._logger.trace(__name__, f"Finished command realm list")

    @realm.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def add(self, ctx: Context, name: str, leader: Optional[discord.Member] = None):
        self._logger.debug(__name__, f"Received command realm list {ctx}")

        if self._realm_utils.find_role_by_name(ctx.guild, name) is not None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.role_exists").format(name),
            )
            return

        if name in [x.name for x in ctx.guild.categories]:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.category_exists").format(name),
            )
            return

        try:
            role = await ctx.guild.create_role(name=name)
            server = self._servers.get_server_by_discord_id(ctx.guild.id)
            if self._realms.find_realm_by_name(name) is not None:
                await role.delete()
                await self._message_service.send_ctx_msg(
                    ctx,
                    self._t.transform("modules.realm.add.error.realm_exists").format(name),
                )
                return

            realm = Realm(name, role.id, server)
            self._realms.add_realm(realm)
            realm = self._realms.find_realm_by_name(name)
            if realm is None:
                raise ValueError("Realm not found after creation")

            if leader is not None:
                self._realms.add_realm_moderator(
                    RealmModerator(realm, self._users.get_user_by_discord_id_and_server_id(leader.id, server.id))
                )

            await self._realm_utils.create_category(ctx.guild, name, role, leader)

            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.success").format(name),
            )
        except Exception as e:
            role = self._realm_utils.find_role_by_name(ctx.guild, name)
            if role is not None:
                await role.delete()

            self._logger.error(__name__, f"Error creating realm: {e}")
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.realm_create").format(name),
            )
            return

        self._logger.trace(__name__, f"Finished realm list command")

    @realm.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def edit(self, ctx: Context, realm_name: str, name: str):
        self._logger.debug(__name__, f"Received command realm list {ctx}")
        realm = self._realms.find_realm_by_name(realm_name)
        if realm is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.edit.error.realm_not_found").format(realm_name),
            )
            return

        realm.name = name
        self._realms.update_realm(realm)

        role = self._realm_utils.find_role_by_name(ctx.guild, realm_name)
        if role is None and self._realm_utils.find_role_by_name(ctx.guild, name) is None:
            role = await ctx.guild.create_role(name=name)
            realm.role_id = role.id
            self._realms.update_realm(realm)

        elif role is not None:
            await role.edit(name=name)

        category = self._realm_utils.find_category_by_name(ctx.guild, realm_name)
        if category is None:
            mods = self._realms.get_realm_moderators_by_realm_id(realm.id)
            leader = mods[0].user.discord_id if len(mods) > 0 else None

            await self._realm_utils.create_category(ctx.guild, name, role, leader)

        await category.edit(name=name)
        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.realm.edit.success").format(realm_name, name),
        )

        self._logger.trace(__name__, f"Finished realm list command")

    @edit.autocomplete("realm_name")
    async def remove_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        return await self._auto_realm_complete(interaction, current)

    @realm.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def delete(self, ctx: Context, realm_name: str):
        self._logger.debug(__name__, f"Received command realm list {ctx}")
        realm = self._realms.find_realm_by_name(realm_name)
        if realm is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.delete.error.realm_not_found").format(realm_name),
            )
            return

        role = self._realm_utils.find_role_by_name(ctx.guild, realm_name)
        if role is not None:
            await role.delete()

        category = self._realm_utils.find_category_by_name(ctx.guild, realm_name)
        if category is not None:
            for c in category.channels:
                await c.delete()

            await category.delete()

        mods = self._realms.get_realm_moderators_by_realm_id(realm.id)
        for mod in mods:
            self._realms.delete_realm_moderator(mod)

        self._realms.delete_realm(realm)
        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.realm.delete.success").format(realm_name),
        )

        self._logger.trace(__name__, f"Finished realm list command")

    @delete.autocomplete("realm_name")
    async def remove_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        return await self._auto_realm_complete(interaction, current)

    @realm.group(name="member")
    @commands.guild_only()
    async def member(self, ctx: Context):
        pass

    @member.command("add")
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    async def member_add(self, ctx: Context, realm: str, member: Member):
        self._logger.debug(__name__, f"Received command realm member add {ctx}")

        realm = self._realms.find_realm_by_name(realm)
        if realm is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.realm_not_found").format(realm),
            )
            return

        if not self._permissions.is_member_moderator(ctx.author) and not self._realm_utils.is_realm_moderator(
            ctx.author.id, realm.id
        ):
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.error.not_moderator").format(realm.name),
            )
            return

        role = self._realm_utils.find_role_by_name(ctx.guild, realm.name)
        if role is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.role_not_found").format(realm.name),
            )
            return

        await member.add_roles(role)
        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.realm.member.add.success").format(member.name, realm.name),
        )

        self._logger.trace(__name__, f"Finished realm list command")

    @member_add.autocomplete("realm")
    async def add_autocomplete(self, interaction: discord.Interaction, current: str) -> TList[app_commands.Choice[str]]:
        return await self._auto_realm_complete(interaction, current)

    @member.command("remove")
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    async def member_remove(self, ctx: Context, realm: str, member: Member):
        self._logger.debug(__name__, f"Received command realm member remove {ctx}")

        realm = self._realms.find_realm_by_name(realm)
        if realm is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.realm_not_found").format(realm),
            )
            return

        if not self._permissions.is_member_moderator(ctx.author) and not self._realm_utils.is_realm_moderator(
            ctx.author.id, realm.id
        ):
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.error.not_moderator").format(realm.name),
            )
            return

        role = self._realm_utils.find_role_by_name(ctx.guild, realm.name)
        if role is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.role_not_found").format(realm.name),
            )
            return

        await member.remove_roles(role)
        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.realm.member.remove.success").format(member.name, realm.name),
        )

        self._logger.trace(__name__, f"Finished realm remove command")

    @member_remove.autocomplete("realm")
    async def remove_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        return await self._auto_realm_complete(interaction, current)

    @realm.group(name="moderator")
    @commands.guild_only()
    async def moderator(self, ctx: Context):
        pass

    @moderator.command("add")
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    async def moderator_add(self, ctx: Context, realm: str, moderator: Member):
        self._logger.debug(__name__, f"Received command realm moderator add {ctx}")

        realm = self._realms.find_realm_by_name(realm)
        if realm is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.realm_not_found").format(realm),
            )
            return

        if not self._permissions.is_member_moderator(ctx.author) and not self._realm_utils.is_realm_moderator(ctx.author.id, realm.id):
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.error.not_moderator").format(realm.name),
            )
            return

        role = self._realm_utils.find_role_by_name(ctx.guild, realm.name)
        if role is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.role_not_found").format(realm.name),
            )
            return

        category = self._realm_utils.find_category_by_name(ctx.guild, realm.name)
        await self._realm_utils.assign_moderator(category, moderator)
        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.realm.moderator.add.success").format(moderator.name, realm.name),
        )

        self._logger.trace(__name__, f"Finished realm list command")

    @moderator_add.autocomplete("realm")
    async def add_autocomplete(self, interaction: discord.Interaction, current: str) -> TList[app_commands.Choice[str]]:
        return await self._auto_realm_complete(interaction, current)

    @moderator.command("remove")
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    async def moderator_remove(self, ctx: Context, realm: str, moderator: Member):
        self._logger.debug(__name__, f"Received command realm moderator remove {ctx}")

        realm = self._realms.find_realm_by_name(realm)
        if realm is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.realm_not_found").format(realm),
            )
            return

        if not self._permissions.is_member_moderator(ctx.author) and not self._realm_utils.is_realm_moderator(ctx.author.id, realm.id):
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.error.not_moderator").format(realm.name),
            )
            return

        role = self._realm_utils.find_role_by_name(ctx.guild, realm.name)
        if role is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.role_not_found").format(realm.name),
            )
            return

        category = self._realm_utils.find_category_by_name(ctx.guild, realm.name)
        if category is None:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("modules.realm.add.error.category_not_found").format(realm.name),
            )
            return

        await self._realm_utils.remove_moderator(category, moderator)
        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.realm.moderator.remove.success").format(moderator.name, realm.name),
        )

        self._logger.trace(__name__, f"Finished realm remove command")

    @moderator_remove.autocomplete("realm")
    async def remove_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        return await self._auto_realm_complete(interaction, current)
